#ifndef CONNECTION_H
#define CONNECTION_H

#include <QObject>
#include <QStringList>
#include <QVariant>
#include <QDateTime>
#include <QByteArray>
#include <QTextCodec>
#include <cups/cups.h>
#include <cups/http.h>
#include <cups/ipp.h>
#include "ipprequest.h"
#include "ippresponse.h"
#include "job.h"
#include "fly-admin-printer_export.h"

class SubscriptionChecker;

namespace Cups
{

class FLY_ADMIN_PRINTER_EXPORT Connection : public QObject
{
    Q_OBJECT
public:
    enum JobCompletion
    {
        All = -1,
        Active = 0,
        Completed = 1
    };

    http_t *handle() const;

    void reconnect(bool force = true);
    //constructor
    static Connection *instance();

    QHash<QString, QString> cupsAdminGetServerSettings();
    bool cupsAdminSetServerSettings(const QHash<QString, QString> &settings);
    void setTimeout(double timeout);
    bool isNull() const;
    void lock();
    void unlock();
    bool isLocked() const;
    void addUnlockCall(QObject *obj, char *) const;
    QList <QVariantMap> destinations();
    QVariantMap destAttrs(const QString &uri);
    bool setPrinterPolicy(const QString &uri, const QString &policy);
    bool setPrinterMac(const QString &uri, const QString &min, const QString &max);
    QList < QMap <QString, QString> > cupsDestinations(const QString &name = QString());
    QList <Job> jobs(const QString &dest = QString(), bool myjobs = false, JobCompletion completed = Active);
    QVariantMap jobAttrs(const QString &uri, int id);
    QList <QVariantMap> devices();
    QList <QVariantMap> PPDs(const QString &deviceId = QString());
    ipp_pstate_t printerState(const QString &name);
    ipp_pstate_t classState(const QString &name);
    bool resumePrinter(const QString &uri);
    bool pausePrinter(const QString &uri);
    bool acceptJobs(const QString &uri);
    bool rejectJobs(const QString &uri, const QString &reason);
    bool printerSetServerDefault(const QString &uri);
    bool addPrinter(const QString &uri, const QString &name, const QString &ppd, const QString &location = QString(), const QString &description = QString(), bool shared = false);
    bool addClass(const QString &name, const QStringList &uris, const QString &location = QString(), const QString &description = QString(), bool shared = false);
    bool deleteClass(const QString &uri);
    bool deletePrinter(const QString &uri);
    bool holdJob(const QString &uri, int id);
    bool releaseJob(const QString &uri, int id);
    bool cancelJob(const QString &dest, int id);
    bool purgeJob(const QString &dest, int id);
    bool purgeJobs(const QString &dest);
    bool restartJob(const QString &dest, int id);
    bool authenticateJob(const QString &dest, int id, const QStringList &authInfoValues);
    int subscribe(int leaseDuration);
    bool moveJob(const QString &jobUri, const QString &destUri);
    QList<int> getSubscriptions();
    bool renewSubscription(int id);
    bool cancelSubscription(int id);
    bool hasSubscription() const;
    static QString lastErrorString();
    static bool isLocal();

signals:
    void unlocked();

private:
    Connection(http_t *http);
    ~Connection();

    static Connection* create(const QString &host, int port, http_encryption_t encryption);
    static Connection* create();

    ipp_pstate_t destState(const QString &uri);
    QVariant getAttribute(const QVariantMap &data, const QString &key, const QVariant &defaultValue = QVariant());

    http_t *m_http = nullptr;
    SubscriptionChecker *m_subscriptionChecker = nullptr;
    bool m_locked = false;
    static Connection *m_instance;

};

} //namespace

#endif//CONNECTION_H
