#ifndef __JOBWIDGET_H
#define __JOBWIDGET_H

#include <QTreeWidget>
#include <QPointer>
#include <QHash>
#include "fly-admin-printer_export.h"

class QAction;
class QMenu;
class QTimer;
class SecurityJobAction;

namespace Cups
{
class Connection;
class Job;
}

class FLY_ADMIN_PRINTER_EXPORT JobWidget: public QTreeWidget
{
    Q_OBJECT
public:
    enum Action {
        HoldReleaseAction = 0,
        CancelAction,
        RestartAction,
        AuthAction,
        MoveAction,
        PurgeAction,
        ShowCompletedAction,
        ShowCanceledAction,
        ShowAbortedAction,
        ShowMyJobsOnlyAction,
        ShowAttributesAction
    };

    explicit JobWidget(QWidget *parent = nullptr);
    ~JobWidget();

    void load(Cups::Connection *connection, const QString &printer = QString(), bool isClass = false);
    QAction *action(Action id) const;
    QList<QAction *> extraActions() const;

public slots:
    void reload();

protected:
    void contextMenuEvent(QContextMenuEvent *event) override;
    void keyPressEvent(QKeyEvent *event) override;

protected slots:
    void onJobState(const QString &text, const QString &uri, const QString &name);

private slots:
    void cancelJob();
    void purgeJob();
    void holdReleaseJob();
    void restartJob();
    void authenticateJob();
    void moveJob();
    void showAttributes();
    void updateActions();

signals:
    void jobsStateChanged();

private:
    QList<Action> supportedActions(const QTreeWidgetItem *it, bool single) const;
    QList<QTreeWidgetItem *> filterByAction(const QList<QTreeWidgetItem *> &items, Action act) const;
    void connectToDBus();
    void disconnectFromDBus();
    void configureItem(QTreeWidgetItem *item, const Cups::Job &job, const QList <QVariantMap> &dests, bool update);
    QTimer *m_refreshTimer;
    QList<SecurityJobAction*> m_extraActions;
    Cups::Connection *m_connection = nullptr;
    QString m_printer;
    QMenu *m_menu;
    QHash<Action, QAction *> m_actions;
    bool m_isClass = false;
    static const QList<JobWidget::Action> m_actionsToCheck;
};

#endif
